
/**
 * Bounded Buffer.
 * @author BinHan
 */
public class BoundBuffer {
	
	private static final int NAP_TIME = 5;
	private static final int BUFFER_SIZE = 5;
	
	private int iCount, iIn, iOut;
	private Object[] oBuffer;
	
	/**
	 * Constructor, buffer is initially empty.
	 */
	public BoundBuffer()
	{
		this.iCount = 0;
		this.iIn = 0;
		this.iOut = 0;
		this.oBuffer = new Object[BUFFER_SIZE];
	}
	
	/**
	 * Produced and consumer will call this to nap.
	 */
	public static void napping()
	{
		int piSleepTime = (int)(NAP_TIME * Math.random());
		try
		{
			Thread.sleep(piSleepTime*1000);
		}
		catch(InterruptedException e){}
	}
	
	/**
	 * The producer calls the enter() method.
	 * @param poItem
	 */
	public synchronized void enter(Object poItem)
	{
		while(this.iCount == BUFFER_SIZE)
		{
			try
			{
				wait();
			}
			catch (InterruptedException e){}
		}
		
//		add an item to the buffer
		++this.iCount;
		this.oBuffer[this.iIn] = poItem;
		this.iIn = (this.iIn + 1) % BUFFER_SIZE;
		
		if(this.iCount == BUFFER_SIZE)
			System.out.println("Producer Entered, Buffer FULL");
		else
			System.out.println("Producer Entered, Buffer size = " + this.iCount);
		
		notify();
	}
	
	/**
	 * The consumer calls the remove() method.
	 * @return
	 */
	public synchronized Object remove()
	{
		Object poItem;
		
		while(this.iCount == 0)
		{
			try
			{
				wait();
			}
			catch (InterruptedException e){}
		}
		
//		remove an item from the buffer
		--this.iCount;
		poItem = this.oBuffer[this.iOut];
		this.iOut = (this.iOut + 1) % BUFFER_SIZE;
		
		if (this.iCount == 0)
			System.out.println("Consumer consumed, Buffer EMPTY.");
		else
			System.out.println("Consumer consumed, Buffer size = " + this.iCount);
		
		notify();
		
		return poItem;
	}
}

//EOF
